@@ -3,7 +3,6 @@ require File.dirname(__FILE__) + '/../../spec_helper'
 module Spec
   module Mocks
     describe Mock do
-
       before(:each) do
         @mock = mock("test mock")
       end
@@ -23,6 +22,18 @@ module Spec
         end
       end
       
+      it "should report line number of expectation of unreceived message after #should_receive after similar stub" do
+        @mock.stub!(:wont_happen)
+        expected_error_line = __LINE__; @mock.should_receive(:wont_happen).with("x", 3)
+        begin
+          @mock.rspec_verify
+          violated
+        rescue MockExpectationError => e
+          # NOTE - this regexp ended w/ $, but jruby adds extra info at the end of the line
+          e.backtrace[0].should match(/#{File.basename(__FILE__)}:#{expected_error_line}/)
+        end
+      end
+      
       it "should pass when not receiving message specified as not to be received" do
         @mock.should_not_receive(:not_expected)
         @mock.rspec_verify
@@ -37,18 +48,16 @@ module Spec
       
       it "should fail when receiving message specified as not to be received" do
         @mock.should_not_receive(:not_expected)
-        @mock.not_expected
         lambda {
-          @mock.rspec_verify
+          @mock.not_expected
           violated
-        }.should raise_error(MockExpectationError, "Mock 'test mock' expected :not_expected with (any args) 0 times, but received it once")
+        }.should raise_error(MockExpectationError, "Mock 'test mock' expected :not_expected with (no args) 0 times, but received it once")
       end
       
       it "should fail when receiving message specified as not to be received with args" do
         @mock.should_not_receive(:not_expected).with("unexpected text")
-        @mock.not_expected("unexpected text")
         lambda {
-          @mock.rspec_verify
+          @mock.not_expected("unexpected text")
           violated
         }.should raise_error(MockExpectationError, "Mock 'test mock' expected :not_expected with (\"unexpected text\") 0 times, but received it once")
       end
@@ -77,7 +86,7 @@ module Spec
         @mock.rspec_verify
       end
       
-      it "should raise exception if args dont match when method called" do
+      it "should raise exception if args don't match when method called" do
         @mock.should_receive(:something).with("a","b","c").and_return("booh")
         lambda {
           @mock.something("a","d","c")
@@ -85,6 +94,24 @@ module Spec
         }.should raise_error(MockExpectationError, "Mock 'test mock' expected :something with (\"a\", \"b\", \"c\") but received it with (\"a\", \"d\", \"c\")")
       end
            
+      it "should raise exception if args don't match when method called even when the method is stubbed" do
+        @mock.stub!(:something)
+        @mock.should_receive(:something).with("a","b","c")
+        lambda {
+          @mock.something("a","d","c")
+          @mock.rspec_verify
+        }.should raise_error(MockExpectationError, "Mock 'test mock' expected :something with (\"a\", \"b\", \"c\") but received it with (\"a\", \"d\", \"c\")")
+      end
+           
+      it "should raise exception if args don't match when method called even when using null_object" do
+        @mock = mock("test mock", :null_object => true)
+        @mock.should_receive(:something).with("a","b","c")
+        lambda {
+          @mock.something("a","d","c")
+          @mock.rspec_verify
+        }.should raise_error(MockExpectationError, "Mock 'test mock' expected :something with (\"a\", \"b\", \"c\") but received it with (\"a\", \"d\", \"c\")")
+      end
+           
       it "should fail if unexpected method called" do
         lambda {
           @mock.something("a","b","c")
@@ -110,23 +137,19 @@ module Spec
       end
         
       it "should fail right away when method defined as never is received" do
-        pending "Used to pass (false positive). Which one is wrong, the spec or the actual behavior?"
-
         @mock.should_receive(:not_expected).never
         lambda {
           @mock.not_expected
-        }.should raise_error(MockExpectationError, "Mock 'test mock' expected :not_expected 0 times, but received it 1 times")
+        }.should raise_error(MockExpectationError, "Mock 'test mock' expected :not_expected with (no args) 0 times, but received it once")
       end
       
       it "should eventually fail when method defined as never is received" do
         @mock.should_receive(:not_expected).never
-        @mock.not_expected
-          
         lambda {
-          @mock.rspec_verify
-        }.should raise_error(MockExpectationError, "Mock 'test mock' expected :not_expected with (any args) 0 times, but received it once")
+          @mock.not_expected
+        }.should raise_error(MockExpectationError, "Mock 'test mock' expected :not_expected with (no args) 0 times, but received it once")
       end
-
+    
       it "should raise when told to" do
         @mock.should_receive(:something).and_raise(RuntimeError)
         lambda do
@@ -186,18 +209,24 @@ module Spec
           @mock.something 1
         }.should raise_error(MockExpectationError, "Mock 'test mock' expected :something with (no args) but received it with (1)")
       end
-
+    
       it "should fail when args are expected but none are received" do
         @mock.should_receive(:something).with(1)
         lambda {
           @mock.something
         }.should raise_error(MockExpectationError, "Mock 'test mock' expected :something with (1) but received it with (no args)")
       end
-      
+    
+      it "should return value from block by default" do
+        @mock.stub!(:method_that_yields).and_yield
+        @mock.method_that_yields { :returned_obj }.should == :returned_obj
+        @mock.rspec_verify
+      end
+    
       it "should yield 0 args to blocks that take a variable number of arguments" do
         @mock.should_receive(:yield_back).with(no_args()).once.and_yield
         a = nil
-        @mock.yield_back {|*a|}
+        @mock.yield_back {|*x| a = x}
         a.should == []
         @mock.rspec_verify
       end
@@ -215,7 +244,7 @@ module Spec
       it "should yield one arg to blocks that take a variable number of arguments" do
         @mock.should_receive(:yield_back).with(no_args()).once.and_yield(99)
         a = nil
-        @mock.yield_back {|*a|}
+        @mock.yield_back {|*x| a = x}
         a.should == [99]
         @mock.rspec_verify
       end
@@ -234,11 +263,11 @@ module Spec
       it "should yield many args to blocks that take a variable number of arguments" do
         @mock.should_receive(:yield_back).with(no_args()).once.and_yield(99, 27, "go")
         a = nil
-        @mock.yield_back {|*a|}
+        @mock.yield_back {|*x| a = x}
         a.should == [99, 27, "go"]
         @mock.rspec_verify
       end
-
+    
       it "should yield many args 3 times consecutively to blocks that take a variable number of arguments" do
         @mock.should_receive(:yield_back).once.with(no_args()).once.and_yield(99, :green, "go").
                                                                     and_yield("wait", :amber).
@@ -253,7 +282,7 @@ module Spec
       it "should yield single value" do
         @mock.should_receive(:yield_back).with(no_args()).once.and_yield(99)
         a = nil
-        @mock.yield_back {|a|}
+        @mock.yield_back {|x| a = x}
         a.should == 99
         @mock.rspec_verify
       end
@@ -272,7 +301,7 @@ module Spec
       it "should yield two values" do
         @mock.should_receive(:yield_back).with(no_args()).once.and_yield('wha', 'zup')
         a, b = nil
-        @mock.yield_back {|a,b|}
+        @mock.yield_back {|x,y| a=x; b=y}
         a.should == 'wha'
         b.should == 'zup'
         @mock.rspec_verify
@@ -347,7 +376,7 @@ module Spec
         mock.rspec_reset
         mock.rspec_verify #should throw if reset didn't work
       end
-
+    
       it "should work even after method_missing starts raising NameErrors instead of NoMethodErrors" do
         # Object#method_missing throws either NameErrors or NoMethodErrors.
         #
@@ -376,7 +405,7 @@ module Spec
         lambda { @mock.foobar }.should_not raise_error(NameError)
         lambda { @mock.foobar }.should raise_error(MockExpectationError)
       end
-
+    
       it "should temporarily replace a method stub on a mock" do
         @mock.stub!(:msg).and_return(:stub_value)
         @mock.should_receive(:msg).with(:arg).and_return(:mock_value)
@@ -386,6 +415,21 @@ module Spec
         @mock.rspec_verify
       end
 
+      it "should not require a different signature to replace a method stub" do
+        @mock.stub!(:msg).and_return(:stub_value)
+        @mock.should_receive(:msg).and_return(:mock_value)
+        @mock.msg(:arg).should equal(:mock_value)
+        @mock.msg.should equal(:stub_value)
+        @mock.msg.should equal(:stub_value)
+        @mock.rspec_verify
+      end
+
+      it "should raise an error when a previously stubbed method has a negative expectation" do
+        @mock.stub!(:msg).and_return(:stub_value)
+        @mock.should_not_receive(:msg).and_return(:mock_value)
+        lambda {@mock.msg(:arg)}.should raise_error(MockExpectationError)
+      end
+
       it "should temporarily replace a method stub on a non-mock" do
         non_mock = Object.new
         non_mock.stub!(:msg).and_return(:stub_value)
@@ -395,13 +439,39 @@ module Spec
         non_mock.msg.should equal(:stub_value)
         non_mock.rspec_verify
       end
-      
+
+      it "should return the stubbed value when no new value specified" do
+        @mock.stub!(:msg).and_return(:stub_value)
+        @mock.should_receive(:msg)
+        @mock.msg.should equal(:stub_value)
+        @mock.rspec_verify
+      end
+
+      it "should not mess with the stub's yielded values when also mocked" do
+        @mock.stub!(:yield_back).and_yield(:stub_value)
+        @mock.should_receive(:yield_back).and_yield(:mock_value)
+        @mock.yield_back{|v| v.should == :mock_value }
+        @mock.yield_back{|v| v.should == :stub_value }
+        @mock.rspec_verify
+      end
+
+      it "should yield multiple values after a similar stub" do
+        File.stub!(:open).and_yield(:stub_value)
+        File.should_receive(:open).and_yield(:first_call).and_yield(:second_call)
+        yielded_args = []
+        File.open {|v| yielded_args << v }
+        yielded_args.should == [:first_call, :second_call]
+        File.open {|v| v.should == :stub_value }
+        File.rspec_verify
+      end
+
       it "should assign stub return values" do
         mock = Mock.new('name', :message => :response)
         mock.message.should == :response
       end
+      
     end
-
+    
     describe "a mock message receiving a block" do
       before(:each) do
         @mock = mock("mock")
@@ -414,62 +484,71 @@ module Spec
       
       it "should call the block after #should_receive" do
         @mock.should_receive(:foo) { add_call }
-
+    
         @mock.foo
-
+    
         @calls.should == 1
       end
-
+    
+      it "should call the block after #should_receive after a similar stub" do
+        @mock.stub!(:foo).and_return(:bar)
+        @mock.should_receive(:foo) { add_call }
+    
+        @mock.foo
+    
+        @calls.should == 1
+      end
+    
       it "should call the block after #once" do
         @mock.should_receive(:foo).once { add_call }
-
+    
         @mock.foo
-
+    
         @calls.should == 1
       end
-
+    
       it "should call the block after #twice" do
         @mock.should_receive(:foo).twice { add_call }
-
+    
         @mock.foo
         @mock.foo
-
+    
         @calls.should == 2
       end
-
+    
       it "should call the block after #times" do
         @mock.should_receive(:foo).exactly(10).times { add_call }
         
         (1..10).each { @mock.foo }
-
+    
         @calls.should == 10
       end
-
+    
       it "should call the block after #any_number_of_times" do
         @mock.should_receive(:foo).any_number_of_times { add_call }
         
         (1..7).each { @mock.foo }
-
+    
         @calls.should == 7
       end
-
-      it "should call the block after #with" do
-        @mock.should_receive(:foo).with(:arg) { add_call }
-        
-        @mock.foo(:arg)
-
-        @calls.should == 1
-      end
-
+    
       it "should call the block after #ordered" do
         @mock.should_receive(:foo).ordered { add_call }
         @mock.should_receive(:bar).ordered { add_call }
         
         @mock.foo
         @mock.bar
-
+    
         @calls.should == 2
       end
     end
+    
+    describe 'string representation generated by #to_s' do
+      it 'should not contain < because that might lead to invalid HTML in some situations' do
+        mock = mock("Dog")
+        valid_html_str = "#{mock}"
+        valid_html_str.should_not include('<')
+      end
+    end
   end
 end
